@@ -3,7 +3,7 @@ import tempfile
 from copy import copy
 from dataclasses import asdict, dataclass
 from pathlib import Path
-from typing import Dict, List, Literal, Optional, Set, Tuple, Union, cast
+from typing import Callable, Dict, List, Literal, Optional, Set, Tuple, Union, cast
 
 import matplotlib.pyplot as plt
 import orjson
@@ -101,7 +101,7 @@ class DataNode(BaseNode[int]):
 
 class LiteralNodeDataType(str, enum.Enum):
     String = "string"
-    # should use full URI
+    # although the string is entity-id, the expected value is the entity's full URI
     Entity = "entity-id"
 
 
@@ -202,6 +202,14 @@ class SemanticModel(RetworkXDiGraph[str, Node, Edge]):
             self.value2id[node.value] = node_id
         return node_id
 
+    def remove_node(self, node_id: int):
+        node = self._graph.get_node_data(node_id)
+        if isinstance(node, DataNode):
+            self.column2id[node.col_index] = -1
+        elif isinstance(node, LiteralNode):
+            del self.value2id[node.value]
+        return super().remove_node(node_id)
+
     def get_semantic_types_of_column(self, col_index: int) -> List[SemanticType]:
         dnode = self.get_data_node(col_index)
         sem_types = set()
@@ -364,6 +372,24 @@ class SemanticModel(RetworkXDiGraph[str, Node, Edge]):
             record = orjson.loads(f.read())
             return SemanticModel.from_dict(record)
 
+    def add_readable_label(
+        self, fn: Callable[[Union[ClassNode, LiteralNode, Edge]], None]
+    ):
+        """Add readable label to all nodes and edges that don't have one yet.
+
+        Note: this function will mutate the semantic model.
+        """
+        for node in self.iter_nodes():
+            if (
+                isinstance(node, (ClassNode, LiteralNode))
+                and node.readable_label is None
+            ):
+                fn(node)
+        for edge in self.iter_edges():
+            if edge.readable_label is None:
+                fn(edge)
+        return self
+
     def draw(
         self,
         filename: Optional[str] = None,
